@@ -9,5 +9,5 @@ use scheduler::JobStatus;
 
 pub use job::Job;
 pub use scheduler::JobScheduler;
-pub use status_server::StatusServer;
+pub use status_server::{State, StatusServer};
 pub use task_manager::{JobManager, TaskManager, TaskResourceHandle};
